/*
 * CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL-B
 * https://cecill.info/licences/Licence_CeCILL-B_V1-fr.html
 */
package fr.unistra.sil.erp.back.db;

import fr.unistra.sil.erp.back.model.Category;
import fr.unistra.sil.erp.back.model.Item;
import fr.unistra.sil.erp.back.model.Stock;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * SQLite implementation of the Database interface.
 * @author BEAUVAIS ANTOINE <antoine.beauvais@etu.unistra.fr>
 */
public class DatabaseSQLiteImpl implements DatabaseInterface {
    
    private static final String CONNECTION_URL = "jdbc:sqlite:dev.db";
    private static final String SQL_GETALLITEMS = 
            "SELECT ref, name, price, subscriberPrice, category FROM items";
    
    private static final String SQL_GETITEMSFROMCATEGORY = SQL_GETALLITEMS +
            " WHERE category = ?";
    
    private static final String SQL_GETCATEGORIES =
            "SELECT id, name FROM categories";
    
    private static final String SQL_GETSTOCKS = 
            "SELECT s.id AS id, s.item AS item, " +
            "i.name AS name, s.quantity AS quantity FROM stocks s " +
            "INNER JOIN items i ON s.item = i.ref";
    
    private Connection conn;
    
    public DatabaseSQLiteImpl()
    {
        try {
            conn = DriverManager.getConnection(CONNECTION_URL);
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to connect to SQLite file.", ex);
        }
    }
    
    private ResultSet query(String query)
    {
        Statement stmt;
        ResultSet rs;
        try {
            stmt = this.conn.createStatement();
            rs = stmt.executeQuery(query);
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to run query: " + query, ex);
            return null;
        }
        
        return rs;
    }

    @Override
    public List<Item> getAllItems() {
        ResultSet rs = query(SQL_GETALLITEMS);
        if(rs == null)
            return null;
        
        List<Item> res = new ArrayList<>();
        try {
            while(rs.next())
            {
                Item item = new Item(rs.getInt("ref"), rs.getString("name"),
                                     rs.getBigDecimal("price"),
                                     rs.getBigDecimal("subscriberPrice"),
                                     rs.getInt("category"));
                res.add(item);
            }
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, null, ex);
            return null;
        }
        
        return res;
    }

    @Override
    public List<Item> getItemsFromCategory(int category) {
        
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(
                    SQL_GETITEMSFROMCATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }
        
        try {
            ps.setInt(1, category);
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }
        
        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to fetch items for category " +
                            category, ex);
            return null;
        }
        
        List<Item> res = new ArrayList<>();
        try {
            while(rs.next())
            {
                Item item = new Item(rs.getInt("ref"), rs.getString("name"),
                                     rs.getBigDecimal("price"),
                                     rs.getBigDecimal("subscriberPrice"),
                                     rs.getInt("category"));
                res.add(item);
            }
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
        }
        
        return res;
    }

    @Override
    public List<Category> getCategories() {
        ResultSet rs = this.query(SQL_GETCATEGORIES);
        if(rs == null)
            return null;
        
        List<Category> res = new ArrayList<>();
        try {
            while(rs.next())
            {
                Category c = new Category(rs.getInt("id"), rs.getString("name"));
                res.add(c);
            }
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }
        
        return res;
    }

    @Override
    public List<Stock> getStocks() {
        ResultSet rs = this.query(SQL_GETSTOCKS);
        if(rs == null)
            return null;
        
        List<Stock> res = new ArrayList<>();
        try {
            while(rs.next())
            {
                Stock s = new Stock(rs.getInt("id"),
                        rs.getInt("item"), rs.getString("name"),
                        rs.getInt("quantity"));
                res.add(s);
            }
        } catch (SQLException ex) {
            Logger.getLogger(DatabaseSQLiteImpl.class.getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }
        
        return res;
    }
    
}
